//
//  SynthEG.m
//
//  Created by 成田 拓郎 on 10/05/01.
//  Copyright 2010 lovelyshark. All rights reserved.
//

#import "SynthEG.h"

@implementation SynthEG

//EGの進行具合のスコア
- (float)score
{
	switch (outputMode) {
		case EG_ATTACK_MODE:
		case EG_DECAY_MODE:
		case EG_SUSTAIN_MODE:
			return attackCount;
		case EG_RELEASE_MODE:
			return FLT_MAX * ((1.0f - outputLevel) * 0.25f + 0.5f);
//		case EG_SPACE_MODE:
//		default:
//			return FLT_MAX;
	}
	return FLT_MAX;
}

- (int) outputMode
{
	return outputMode;
}
- (void) setOutputMode:(int)m
{
	switch (m) {
		case EG_ATTACK_MODE:
			outputMode  = EG_ATTACK_MODE;
			attackCount = 0.0f;
			break;
		case EG_DECAY_MODE:
			outputMode = EG_DECAY_MODE;
			break;
		case EG_SUSTAIN_MODE:
			outputMode = EG_SUSTAIN_MODE;
			break;
		case EG_RELEASE_MODE:
			outputMode = EG_RELEASE_MODE;
			break;
		case EG_SPACE_MODE:
		default:
			outputMode = EG_SPACE_MODE;
			break;
	}
}

@synthesize outputLevel;
@synthesize paramAttack;
@synthesize paramDecay;
@synthesize paramSustain;
@synthesize paramRelease;

//このオブジェクトの演算関数
- (void) synthFunc:(int)length
{
	//出力バッファの準備
	float *waveout_dest = outputBuffer;
	float *waveout_end  = waveout_dest + length;
	//int i = 0;

	float outputSampleRate = [SynthObject outputSampleRate:0.0f];

	//Attack
	if (outputMode == EG_ATTACK_MODE) {
		float attack_sample = (paramAttack <= 0.01f) ? 0.01f : paramAttack;
		attack_sample *= outputSampleRate;
		if (attack_sample <= 0.0f) {
			outputLevel = 1.0f;
			outputMode = EG_DECAY_MODE;
		}
		else {
			float level = 1.0f / attack_sample;
			while (waveout_dest != waveout_end) {
				*waveout_dest = outputLevel;
				++waveout_dest;
				outputLevel += level;
				if (outputLevel >= 1.0f) {
					outputLevel = 1.0f;
					outputMode = EG_DECAY_MODE;
					break;
				}				
			}
		}
	}
	//Decay
	if (outputMode == EG_DECAY_MODE) {
		float decay_sample = (paramDecay <= 0.01f) ? 0.01f : paramDecay;
		decay_sample *= outputSampleRate;
		if (decay_sample <= 0.0f) {
			outputLevel = paramSustain;
			outputMode = EG_SUSTAIN_MODE;
		}
		else {
			float level = (paramSustain - 1.0f) / decay_sample;
			while (waveout_dest != waveout_end) {
				*waveout_dest = outputLevel;
				++waveout_dest;
				outputLevel += level;
				if (outputLevel <= paramSustain) {
					outputLevel = paramSustain;
					outputMode = EG_SUSTAIN_MODE;
					break;
				}
			}
		}
	}
	//Sustain
	if (outputMode == EG_SUSTAIN_MODE) {
		outputLevel = paramSustain;
		while (waveout_dest != waveout_end) {
			*waveout_dest = outputLevel;
			++waveout_dest;
		}
	}
	//Release
	if (outputMode == EG_RELEASE_MODE) {
		float release_sample = (paramRelease <= 0.01f) ? 0.01f : paramRelease;
		release_sample *= outputSampleRate;
		if (release_sample > 0.0f) {
			float level = (0.0f - paramSustain) / release_sample;
			if (level > -0.001f) level = -0.001f;
			while (waveout_dest != waveout_end) {
				*waveout_dest = outputLevel;
				++waveout_dest;
				outputLevel += level;
				if (outputLevel <= 0.0f) {
					outputLevel = 0.0f;
					outputMode = EG_SPACE_MODE;
					break;
				}
			}
		}
	}
	//終了
	if (waveout_dest != waveout_end) {
		*waveout_dest = outputLevel;
		++waveout_dest;
		outputLevel = 0.0f;
		while (waveout_dest != waveout_end) {
			*waveout_dest = 0.0f;
			++waveout_dest;
		}
	}
	attackCount += length;
}

- (id) init
{
	if (self = [super init])
	{
		self.outputMode		= EG_SPACE_MODE;
		self.paramAttack    = 0.25f;
		self.paramDecay     = 0.25f;
		self.paramSustain   = 0.75f;
		self.paramRelease   = 1.0f;
	}
	
	return self;
}

- (void) dealloc
{
	[super dealloc];
}


@end
